<?php
namespace App\Factory;

use App\Exception\ParamsException as ParamsException;

class ProductFactory {
    
    private $connection;
    private $util;
    
    public function __construct (\Doctrine\DBAL\Connection $connection,
                                 \App\Util\Util $util) {
        $this->connection = $connection;
        $this->util       = $util;
    }
    
    public function getItensVenda($subsidiaryCode, $cdCaixa , $nrSeqVenda){
        
        $sql = "SELECT I.NRSEQUITVEND, I.NRSEQVENDA, P.NMPRODUTO, I.CDPRODUTO, P.CDARVPROD, P.CDPRODINTE,
                       I.QTPRODVEND, I.VRUNITVEND, (I.VRUNITVEND * I.QTPRODVEND) AS TOTAL , I.VRDESITVEND,
                       I.VRACRITVEND, P.CDCLASFISC, P.CDCEST, II.CDCSTPRODI, II.CDCSTPRODPC, II.CDCFOPPROD,
                       II.VRBCPISCOFINS, II.VRIMPPIS, II.VRIMPCOFINS, II.VRBASECALCICMS, II.VRIMPOPRODIT,
                       A.VRALIQPIS, A.VRALIQCOFINS, A.VRPEALIMPFIS, II.VRALIQFCP, II.VRIMPICMSFCP
                FROM ITEMVENDA I
                    JOIN PRODUTO P
                        ON  I.CDPRODUTO = P.CDPRODUTO
                    JOIN ALIQIMPFIS A
                        ON I.CDPRODUTO = A.CDPRODUTO
                        AND I.CDFILIAL = A.CDFILIAL
                    JOIN ITVENDAIMPOS II
                        ON I.NRSEQUITVEND = II.NRSEQUITVEND
                        AND I.CDFILIAL   = II.CDFILIAL
                        AND I.NRSEQVENDA = II.NRSEQVENDA
                        AND I.CDCAIXA    = II.CDCAIXA
                WHERE I.CDFILIAL = :CDFILIAL
                    AND I.CDCAIXA    = :CDCAIXA
                    AND I.NRSEQVENDA = :NRSEQVENDA
                ";
                
        $params = [
                ':CDFILIAL'   => $subsidiaryCode,
                ':CDCAIXA'    => $cdCaixa,
                ':NRSEQVENDA' => $nrSeqVenda
                ];
        
        $itens = $this->connection->fetchAll($sql, $params);
        
        return $itens;
    }
    
    public function getItens($subsidiaryCode, $dtInicial, $dtFinal){
        
        $sql = "SELECT IV.CDFILIAL, IV.CDCAIXA, IV.NRSEQVENDA, IV.NRSEQUITVEND, IV.QTPRODVEND, 
                       IV.CDPRODUTO, P.NMPRODUTO, IV.DTHRPEDIDO, P.IDTIPOCOMPPROD, P.IDUTILRETPRODACAB  
                    FROM ITEMVENDA IV
                  JOIN PRODUTO P
                    ON IV.CDPRODUTO = P.CDPRODUTO
                        WHERE DTHRPEDIDO BETWEEN to_date(:DATA_INI, 'DD/MM/YYYY hh24:mi:ss') and to_date(:DATA_FIM, 'DD/MM/YYYY hh24:mi:ss')
                        ORDER BY NRSEQVENDA, NRSEQUITVEND
                ";
                
        $params = [
                    ':CDFILIAL'    => $subsidiaryCode,
                    ':DATA_INI'    => $dtInicial,
                    ':DATA_FIM'    => $dtFinal . ' ' . '23:59:59'
            ];
        
        return $this->connection->fetchAll($sql, $params);
    }
    
     public function getItensCombo ($subsidiaryCode, $cdCaixa, $nrSeqVenda, $nrSequItVend){
        
        $sql = "SELECT I.CDFILIAL, I.CDCAIXA, I.NRSEQVENDA, I.NRSEQUITVEND,
                       I.CDPRODUTO, P.NMPRODUTO, I.QTITVENDAEST, P.IDUTILRETPRODACAB
                    FROM ITEMVENDAEST I
                  JOIN PRODUTO P
                    ON I.CDPRODUTO = P.CDPRODUTO
                        WHERE I.CDFILIAL = :CDFILIAL
                            AND I.CDCAIXA = :CDCAIXA
                            AND I.NRSEQVENDA = :NRSEQVENDA
                            AND I.NRSEQUITVEND = :NRSEQUITVEND
                        ORDER BY I.NRSEQVENDA, I.NRSEQUITVEND
                ";
                
        $params = [
                    ':CDFILIAL'     => $subsidiaryCode,
                    ':CDCAIXA'      => $cdCaixa,
                    ':NRSEQVENDA'   => $nrSeqVenda,
                    ':NRSEQUITVEND' => $nrSequItVend
            ];
     
     return $this->connection->fetchAll($sql, $params);   
    }
    
    public function getComposicao($subsidiaryCode, $cdProduto){
        $sqlLocal = "
            SELECT R.CDFILIAL, R.CDPRODUTO, R.CDSUBPRODLOC, P.NMPRODUTO, R.QTSUBPRODLOC, R.NRSEQPRODLOC 
                FROM ROTFABLOCAL R
                JOIN PRODUTO P ON R.CDSUBPRODLOC = P.CDPRODUTO
                    WHERE R.CDFILIAL = :CDFILIAL
                        AND R.CDPRODUTO = :CDPRODUTO
                        
            ORDER BY R.CDPRODUTO, R.NRSEQPRODLOC
        ";
        
        $paramsLocal = [
                    ':CDFILIAL'     => $subsidiaryCode,
                    ':CDPRODUTO'    => $cdProduto
            ];
        
        $localComp = $this->connection->fetchAll($sqlLocal, $paramsLocal);
        
        if(!empty($localComp))
            return $localComp;
            
        $sqlGeral = "
            SELECT R.CDPRODUTO, R.CDSUBPRODUTO, P.NMPRODUTO, R.QTSUBPRODROT, R.NRSEQPROD
                FROM ROTFABRICA R
                 JOIN PRODUTO P ON R.CDSUBPRODUTO = P.CDPRODUTO
                    WHERE R.CDPRODUTO = :CDPRODUTO
                    
            ORDER BY R.CDPRODUTO,R.NRSEQPROD
        ";
        
        $paramsGeral = [
                    ':CDPRODUTO'     => $cdProduto
            ];
        
        $geralComp = $this->connection->fetchAll($sqlGeral, $paramsGeral);
        
        return $geralComp;
    }
    
    public function getCodeProduct ($product){
        
        $sql = "
            SELECT CDPRODINTE, SGUNIDADE
              FROM PRODUTO PR
             WHERE CDPRODUTO  = :CDPRODUTO
                OR CDARVPROD  = :CDARVPROD
        ";

        $productCode = str_replace(".", "", $product);
        $productCode = preg_replace("/[^0-9A-Z]/", "", $productCode);
        

        $params = [
            ':CDPRODUTO'  => $productCode,
            ":CDARVPROD"  => $productCode
        ];
        
        return $this->connection->fetchAssoc($sql, $params);
    }
}